;; currency-converter.scm

;; Copyright 2007 Jean-Philippe Theberge

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or (at
;; your option) any later version.

;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; A full copy of the GPL license can be found at
;; <http://www.gnu.org/licenses/>.

(require-extension ssax http-client format sxml-tools)

(declare (export currency-convert 
		 currency-converter-table 
		 currency-converter-table-initialize!))

(define (cc:conversion-table) 
  ;; convert the sxml data into a alist (curency-symbol . value)
  ;; value is in Euro 
  (cons '(EUR . 1) ;; add euro to the table 1 euro = 1 euro!
	(map (lambda (x)
	       (let ((i (cdadr x)))
		 (cons (string->symbol (cadadr i))
		       (string->number (cadar i)))))
	     ((sxpath "g:Envelope/c:Cube/c:Cube/c:Cube" 
		      '[(g . "http://www.gesmes.org/xml/2002-08-01")
			(c . "http://www.ecb.int/vocabulary/2002-08-01/eurofxref")])
	      ;;  Fetch xml info from the currency server at ecb.int and return sxml  
	      (SSAX:XML->SXML 
	       (open-input-string 
		(http:GET "http://www.ecb.int/stats/eurofxref/eurofxref-daily.xml"))
	       '())))))

(define (cc:money n)
  ;; Round up n into money (ex: 3.2 ==> 3.20 and 3.67899 ==> 3.68)
  ;; Take a number, return a string
  (let ((l (string-split (number->string (* (round (* n 100)) 0.01)) ".")))
    (format #f "~a.~2,,,'0@a" (car l) (cadr l))))

(define cc:ctable #f)

(define (cc:ctable-initialize!)
  (set! cc:ctable (cc:conversion-table)))

(define (cc:convert from to amount . args)  
  (let* ((tbl (or (get-keyword #:table args) cc:ctable (cc:conversion-table)))
	;; just in case the amount is a string
	(a (if (string? amount) (string->number amount) amount))
	(result (* a (/ (alist-ref to tbl) (alist-ref from tbl)))))
    (if (get-keyword #:format? args (lambda () #t)) 
	(cc:money result)
	result)))

(define currency-convert cc:convert)
(define currency-converter-table cc:ctable)
(define currency-converter-table-initialize! cc:ctable-initialize!)
